"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codegen_1 = require("@microsoft.azure/codegen");
const expression_1 = require("./expression");
const namespace_1 = require("./namespace");
const parameter_1 = require("./parameter");
const property_1 = require("./property");
class ClassType {
    constructor(namespace, name) {
        this.name = name;
        this.namespace = typeof (namespace) === 'string' ? new namespace_1.Namespace(namespace) : namespace;
    }
    get fullName() {
        return this.namespace.fullName ? `${this.namespace.fullName}.${this.name}` : this.name;
    }
    get declaration() {
        return `${this.fullName}`;
    }
    toString() {
        return this.declaration;
    }
    new(...parameters) {
        return expression_1.toExpression(`new ${this.fullName}(${parameters.joinWith(expression_1.valueOf)})`);
    }
    Cast(expression) {
        return expression_1.toExpression(`(${this.declaration})${expression}`);
    }
}
exports.ClassType = ClassType;
class EnumType {
    constructor(namespace, name) {
        this.namespace = namespace;
        this.name = name;
    }
    get declaration() {
        return `${this.namespace}.${this.name}`;
    }
    member(enumMember) {
        return new expression_1.LiteralExpression(`${this.declaration}.${enumMember}`);
    }
    get definition() {
        return ``;
    }
    toString() {
        return this.declaration;
    }
    newProperty(name, objectInitializer) {
        return new property_1.Property(name, this, objectInitializer);
    }
    newParameter(name, objectInitializer) {
        return new parameter_1.Parameter(name, this, objectInitializer);
    }
    withMembers() {
        const $this = this;
        return new Proxy($this, {
            get(target, prop, receiver) {
                // return a property if there is one for this.
                if (target[prop]) {
                    return target[prop];
                }
                return $this.member(prop.toString());
            }
        });
    }
}
exports.EnumType = EnumType;
exports.None = new namespace_1.Namespace('');
const system = new namespace_1.Namespace('System');
const threading = new namespace_1.Namespace('Threading', system);
const text = new namespace_1.Namespace('Text', system);
const tasks = new namespace_1.Namespace('Tasks', threading);
const action = new ClassType(system, 'Action');
const collections = new namespace_1.Namespace('Collections', system);
const generic = new namespace_1.Namespace('Generic', collections);
const net = new namespace_1.Namespace('Net', system);
const io = new namespace_1.Namespace('IO', system);
const http = new namespace_1.Namespace('Http', net);
const headers = new namespace_1.Namespace('Headers', http);
const task = new ClassType(tasks, 'Task');
const encoding = new ClassType(text, 'Encoding');
const xml = new namespace_1.Namespace('Xml', system);
const xmllinq = new namespace_1.Namespace('Linq', xml);
const stringClass = new ClassType(system, 'String');
exports.System = codegen_1.intersect(system, {
    Threading: codegen_1.intersect(threading, {
        CancellationToken: new ClassType(threading, 'CancellationToken'),
        CancellationTokenSource: new ClassType(threading, 'CancellationTokenSource'),
        Tasks: codegen_1.intersect(tasks, {
            Task(taskType) {
                return taskType ? new ClassType(tasks, `Task<${taskType.declaration}>`) : task;
            }
        })
    }),
    Object: new ClassType(system, 'Object'),
    String: codegen_1.intersect(stringClass, {
        Empty: new expression_1.LiteralExpression('System.String.Empty'),
        IsNullOrEmpty: (expression) => expression_1.toExpression(`${exports.System.String}.IsNullOrEmpty(${expression_1.toExpression(expression)})`),
        IsNullOrWhitespace: (expression) => expression_1.toExpression(`${exports.System.String}.IsNullOrWhitespace(${expression_1.toExpression(expression)})`),
        /** @description Binds a Variable to the known instance methods of this type */
        $BindTo: ($instance) => {
            return codegen_1.intersect($instance, {
                Substring: (start, end) => {
                    return end ? `${$instance}.Substring(${start},${end})` : `${$instance}.Substring(${start})`;
                }
            });
        },
    }),
    DateTime: new ClassType(system, 'DateTime'),
    EventArgs: new ClassType(system, 'EventArgs'),
    Exception: new ClassType(system, 'Exception'),
    AggregateException: new ClassType(system, 'AggregateException'),
    TimeSpan: new ClassType(system, 'TimeSpan'),
    Type: new ClassType(system, 'Type'),
    Uri: new ClassType(system, 'Uri'),
    IFormatProvider: new ClassType(system, 'IFormatProvider'),
    Xml: codegen_1.intersect(xml, {
        Linq: codegen_1.intersect(xmllinq, {
            XElement: new ClassType(xmllinq, 'XElement'),
            XAttribute: new ClassType(xmllinq, 'XAttribute')
        })
    }),
    IO: codegen_1.intersect(io, {
        Stream: new ClassType(io, 'Stream')
    }),
    Text: codegen_1.intersect(text, {
        Encoding: codegen_1.intersect(encoding, {
            UTF8: new expression_1.LiteralExpression(`${encoding.declaration}.UTF8`)
        })
    }),
    Net: codegen_1.intersect(net, {
        WebProxy: new ClassType(net, 'WebProxy'),
        Http: codegen_1.intersect(http, {
            HttpRequestMessage: new ClassType(http, 'HttpRequestMessage'),
            HttpClient: new ClassType(http, 'HttpClient'),
            HttpClientHandler: new ClassType(http, 'HttpClientHandler'),
            HttpResponseMessage: new ClassType(http, 'HttpResponseMessage'),
            Headers: codegen_1.intersect(headers, {
                HttpHeaders: new ClassType(headers, 'HttpHeaders'),
                HttpResponseHeaders: new ClassType(headers, 'HttpResponseHeaders'),
            }),
            StringContent: new ClassType(http, 'StringContent'),
            StreamContent: new ClassType(http, 'StreamContent'),
            MultipartFormDataContent: new ClassType(http, 'MultipartFormDataContent'),
        }),
        HttpStatusCode: codegen_1.intersect(new ClassType(net, 'HttpStatusCode'), {
            default: new expression_1.LiteralExpression(''),
            100: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Continue'),
            101: new expression_1.LiteralExpression('System.Net.HttpStatusCode.SwitchingProtocols'),
            200: new expression_1.LiteralExpression('System.Net.HttpStatusCode.OK'),
            201: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Created'),
            202: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Accepted'),
            203: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NonAuthoritativeInformation'),
            204: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NoContent'),
            205: new expression_1.LiteralExpression('System.Net.HttpStatusCode.ResetContent'),
            206: new expression_1.LiteralExpression('System.Net.HttpStatusCode.PartialContent'),
            300: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Ambiguous'),
            301: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Moved'),
            302: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Redirect'),
            303: new expression_1.LiteralExpression('System.Net.HttpStatusCode.SeeOther'),
            304: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NotModified'),
            305: new expression_1.LiteralExpression('System.Net.HttpStatusCode.UseProxy'),
            306: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Unused'),
            307: new expression_1.LiteralExpression('System.Net.HttpStatusCode.TemporaryRedirect'),
            400: new expression_1.LiteralExpression('System.Net.HttpStatusCode.BadRequest'),
            401: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Unauthorized'),
            402: new expression_1.LiteralExpression('System.Net.HttpStatusCode.PaymentRequired'),
            403: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Forbidden'),
            404: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NotFound'),
            405: new expression_1.LiteralExpression('System.Net.HttpStatusCode.MethodNotAllowed'),
            406: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NotAcceptable'),
            407: new expression_1.LiteralExpression('System.Net.HttpStatusCode.ProxyAuthenticationRequired'),
            408: new expression_1.LiteralExpression('System.Net.HttpStatusCode.RequestTimeout'),
            409: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Conflict'),
            410: new expression_1.LiteralExpression('System.Net.HttpStatusCode.Gone'),
            411: new expression_1.LiteralExpression('System.Net.HttpStatusCode.LengthRequired'),
            412: new expression_1.LiteralExpression('System.Net.HttpStatusCode.PreconditionFailed'),
            413: new expression_1.LiteralExpression('System.Net.HttpStatusCode.RequestEntityTooLarge'),
            414: new expression_1.LiteralExpression('System.Net.HttpStatusCode.RequestUriTooLong'),
            415: new expression_1.LiteralExpression('System.Net.HttpStatusCode.UnsupportedMediaType'),
            416: new expression_1.LiteralExpression('System.Net.HttpStatusCode.RequestedRangeNotSatisfiable'),
            417: new expression_1.LiteralExpression('System.Net.HttpStatusCode.ExpectationFailed'),
            426: new expression_1.LiteralExpression('System.Net.HttpStatusCode.UpgradeRequired'),
            500: new expression_1.LiteralExpression('System.Net.HttpStatusCode.InternalServerError'),
            501: new expression_1.LiteralExpression('System.Net.HttpStatusCode.NotImplemented'),
            502: new expression_1.LiteralExpression('System.Net.HttpStatusCode.BadGateway'),
            503: new expression_1.LiteralExpression('System.Net.HttpStatusCode.ServiceUnavailable'),
            504: new expression_1.LiteralExpression('System.Net.HttpStatusCode.GatewayTimeout'),
            505: new expression_1.LiteralExpression('System.Net.HttpStatusCode.HttpVersionNotSupported')
        }),
    }),
    Collections: codegen_1.intersect(collections, {
        Hashtable: new ClassType(collections, 'Hashtable'),
        Generic: codegen_1.intersect(generic, {
            Dictionary(keyType, valueType) {
                return new ClassType(generic, `Dictionary<${keyType.declaration},${valueType.declaration}>`);
            },
            KeyValuePair(keyType, valueType) {
                return new ClassType(generic, `KeyValuePair<${keyType.declaration},${valueType.declaration}>`);
            },
            IEnumerable(type) {
                return new ClassType(generic, `IEnumerable<${type.declaration}>`);
            }
        })
    }),
    Action(...actionParameters) {
        return actionParameters.length === 0 ? action : new ClassType(system, `Action<${actionParameters.filter(each => each.declaration).joinWith(each => each.declaration)}>`);
    },
    Func(...funcParameters) {
        return new ClassType(system, `Func<${funcParameters.joinWith(each => each.declaration)}>`);
    }
});
exports.dotnet = {
    Unknown: new ClassType(exports.None, 'null'),
    ToDo: new ClassType(exports.None, 'null'),
    Void: new ClassType(exports.None, 'void'),
    String: new ClassType(exports.None, 'string'),
    Int: new ClassType(exports.None, 'int'),
    Long: new ClassType(exports.None, 'long'),
    Double: new ClassType(exports.None, 'double'),
    Float: new ClassType(exports.None, 'float'),
    Binary: new ClassType(exports.None, 'byte[]'),
    Bool: new ClassType(exports.None, 'bool'),
    Object: new ClassType(exports.None, 'object'),
    Dynamic: new ClassType(exports.None, 'dynamic'),
    ThisObject: new ClassType(exports.None, 'this object'),
    Var: new ClassType(exports.None, 'var'),
    True: new expression_1.LiteralExpression('true'),
    False: new expression_1.LiteralExpression('false'),
    Null: new expression_1.LiteralExpression('null'),
    This: new expression_1.LiteralExpression('this'),
};
/*

const q = System.String.$BindTo(Local('text', '"helloworld"'));
const f = Local('text', `"helloworld"`).As(System.String);

f.Substring('10');

*/
//# sourceMappingURL=dotnet.js.map